@@ -12,8 +12,6 @@ int main(int argc, char *argv[])
 {
     int (*main_func)(int, char **);
     char *error;
-    // 根据参数或环境变量判断启哪个版本
-    char *Compositor = getenv("DDE_CURRENT_COMPOSITOR");
     bool isOldDcc = true;
     if (argc >= 1 && strcmp(argv[argc - 1], "n") == 0) {
         isOldDcc = false;
@@ -21,8 +19,6 @@ int main(int argc, char *argv[])
     } else if (argc >= 1 && strcmp(argv[argc - 1], "o") == 0) {
         isOldDcc = true;
         argc = argc - 1;
-    } else if (Compositor && strcasecmp(Compositor, "treeland") == 0) {
-        isOldDcc = false;
     }
 
     // 加载共享库
