@@ -9,16 +9,19 @@
 #include "pluginmanager.h"
 #include "searchmodel.h"
 
+#include <DGuiApplicationHelper>
+
 #include <QCoreApplication>
 #include <QDBusConnection>
-#include <QDBusMessage>
 #include <QDBusPendingCall>
+#include <QElapsedTimer>
 #include <QJsonArray>
 #include <QJsonDocument>
 #include <QJsonObject>
 #include <QLoggingCategory>
 #include <QQmlApplicationEngine>
 #include <QQmlContext>
+#include <QTimer>
 #include <QTranslator>
 #include <QWindow>
 
@@ -53,7 +56,6 @@ DccManager::DccManager(QObject *parent)
     m_noParentObjects->setName("_noParent");
 
     m_root->setName("root");
-    m_root->setDefultObject(nullptr);
     m_root->setCanSearch(false);
     m_currentObjects.append(m_root);
     onObjectAdded(m_root);
@@ -61,6 +63,7 @@ DccManager::DccManager(QObject *parent)
     initConfig();
     connect(m_plugins, &PluginManager::addObject, this, &DccManager::addObject, Qt::QueuedConnection);
     connect(qApp, &QCoreApplication::aboutToQuit, this, &DccManager::onQuit);
+    showPage("system");
 }
 
 DccManager::~DccManager()
@@ -128,6 +131,17 @@ int DccManager::height() const
     return h > 530 ? h : 530;
 }
 
+DccApp::UosEdition DccManager::uosEdition() const
+{
+    DSysInfo::UosEdition edition = DSysInfo::uosEditionType();
+    return DccApp::UosEdition(edition);
+}
+
+bool DccManager::isTreeland() const
+{
+    return Dtk::Gui::DGuiApplicationHelper::testAttribute(Dtk::Gui::DGuiApplicationHelper::IsWaylandPlatform);
+}
+
 DccObject *DccManager::object(const QString &name)
 {
     return findObject(name);
@@ -149,7 +163,7 @@ void DccManager::addObject(DccObject *obj)
     while (!objs.isEmpty()) {
         DccObject *o = objs.takeFirst();
         if (o->parentName().isEmpty()) {
-            DccObject::Private::FromObject(m_noParentObjects)->addChild(o);
+            DccObject::Private::FromObject(m_noParentObjects)->addChild(o, false);
         } else {
             if (contains(m_hideModule, o)) {
                 DccObject::Private::FromObject(o)->setFlagState(DCC_CONFIG_HIDDEN, true);
@@ -159,9 +173,9 @@ void DccManager::addObject(DccObject *obj)
             }
             if (!o->isVisibleToApp()) {
                 connect(o, &DccObject::visibleToAppChanged, this, &DccManager::onVisible, Qt::ConnectionType(Qt::QueuedConnection | Qt::UniqueConnection));
-                DccObject::Private::FromObject(m_hideObjects)->addChild(o);
+                DccObject::Private::FromObject(m_hideObjects)->addChild(o, false);
             } else if (!addObjectToParent(o)) {
-                DccObject::Private::FromObject(m_noAddObjects)->addChild(o);
+                DccObject::Private::FromObject(m_noAddObjects)->addChild(o, false);
             }
         }
 
@@ -171,7 +185,7 @@ void DccManager::addObject(DccObject *obj)
     objs.append(m_noAddObjects->getChildren());
     while (!objs.isEmpty()) {
         DccObject *o = objs.takeFirst();
-        if (DccObject *parentObj = findObject(o->parentName())) {
+        if (DccObject *parentObj = findParent(o)) {
             DccObject::Private::FromObject(m_noAddObjects)->removeChild(o);
             DccObject::Private::FromObject(parentObj)->addChild(o);
             objs = m_noAddObjects->getChildren();
@@ -193,28 +207,45 @@ void DccManager::removeObject(const QString &name)
 
 void DccManager::showPage(const QString &url)
 {
-    qCInfo(dccLog()) << "show page:" << url;
-    if (url.isEmpty()) {
-        showPage(m_root, QString());
+    if (this->calledFromDBus()) {
+        auto message = this->message();
+        setDelayedReply(true);
+        QMetaObject::invokeMethod(this, &DccManager::waitShowPage, Qt::QueuedConnection, url, message);
+        // show(); // 先查找再显示则注释掉此处
     } else {
-        int i = url.indexOf('.');
-        QString cmd = i != -1 ? url.mid(i + 1, -1) : QString();
-        showPage(findObject(url.mid(0, i)), cmd);
+        QMetaObject::invokeMethod(this, &DccManager::waitShowPage, Qt::QueuedConnection, url, QDBusMessage());
     }
 }
 
+void DccManager::showPage(DccObject *obj)
+{
+    QMetaObject::invokeMethod(this, "doShowPage", Qt::QueuedConnection, obj, QString());
+}
+
 void DccManager::showPage(DccObject *obj, const QString &cmd)
 {
     QMetaObject::invokeMethod(this, "doShowPage", Qt::QueuedConnection, obj, cmd);
 }
 
+void DccManager::toBack()
+{
+    int row = m_navModel->rowCount();
+    if (row < 3) {
+        showPage(m_root);
+    } else {
+        row = row - 3;
+        QString url = m_navModel->data(m_navModel->index(row, 0), NavigationModel::NavUrlRole).toString();
+        if (!url.isEmpty()) {
+            showPage(url);
+        }
+    }
+}
+
 QWindow *DccManager::mainWindow() const
 {
     return m_window;
 }
 
-void DccManager::setShowPath(const QString &path) { }
-
 void DccManager::showHelp()
 {
     QString helpTitle;
@@ -293,6 +324,14 @@ bool DccManager::action(const QString &json)
     return true;
 }
 
+QString DccManager::GetAllModule()
+{
+    auto message = this->message();
+    setDelayedReply(true);
+    QMetaObject::invokeMethod(this, &DccManager::doGetAllModule, Qt::QueuedConnection, message);
+    return QString();
+}
+
 QAbstractItemModel *DccManager::navModel() const
 {
     return m_navModel;
@@ -303,6 +342,14 @@ QSortFilterProxyModel *DccManager::searchModel() const
     return m_searchModel;
 }
 
+void DccManager::show()
+{
+    QWindow *w = mainWindow();
+    if (w->windowStates() == Qt::WindowMinimized || !w->isVisible())
+        w->showNormal();
+    w->requestActivate();
+}
+
 void DccManager::initConfig()
 {
     if (!m_dconfig->isValid()) {
@@ -336,15 +383,20 @@ bool DccManager::isEqual(const QString &url, const DccObject *obj)
     return true;
 }
 
-DccObject *DccManager::findObject(const QString &url)
+DccObject *DccManager::findObject(const QString &url, bool onlyRoot)
 {
+    if (!m_root) {
+        return nullptr;
+    }
     QString path = url;
     if (path.startsWith("/")) {
         path = path.mid(1);
     }
     QVector<QVector<DccObject *>> objs;
     objs.append({ m_root });
-    objs.append(m_hideObjects->getChildren());
+    if (!onlyRoot) {
+        objs.append(m_hideObjects->getChildren());
+    }
     while (!objs.isEmpty()) {
         QVector<DccObject *> subObjs = objs.takeFirst();
         while (!subObjs.isEmpty()) {
@@ -358,11 +410,86 @@ DccObject *DccManager::findObject(const QString &url)
     return nullptr;
 }
 
+DccObject *DccManager::findParent(const DccObject *obj)
+{
+    const QString &path = obj->parentName();
+    DccObject *p = qobject_cast<DccObject *>(obj->parent());
+    if (p) {
+        if (isEqual(path, p)) {
+            return p;
+        }
+        p = qobject_cast<DccObject *>(p->parent());
+        if (p && isEqual(path, p)) {
+            return p;
+        }
+    }
+    return findObject(path);
+}
+
+void DccManager::waitShowPage(const QString &url, const QDBusMessage message)
+{
+    qCInfo(dccLog()) << "show page:" << url;
+    DccObject *obj = nullptr;
+    QString cmd;
+    if (url.isEmpty()) {
+        obj = m_root;
+        showPage(obj, QString());
+    } else {
+        int i = url.indexOf('?');
+        cmd = i != -1 ? url.mid(i + 1) : QString();
+        QString path = url.mid(0, i).split('/', Qt::SkipEmptyParts).join('/'); // 移除多余的/
+        obj = findObject(path, true);
+        if (obj) {
+            showPage(obj, cmd);
+        } else if (!m_plugins->loadFinished()) {
+            QEventLoop loop;
+            QTimer timer;
+            connect(&timer, &QTimer::timeout, this, [this, &path, &loop]() {
+                DccObject *obj = findObject(path, true);
+                if (obj) {
+                    loop.quit();
+                }
+            });
+            connect(m_plugins, &PluginManager::loadAllFinished, &loop, &QEventLoop::quit);
+            timer.start(10);
+            loop.exec();
+            obj = findObject(path, true);
+            if (obj) {
+                showPage(obj, cmd);
+            }
+        }
+    }
+    if (message.type() != QDBusMessage::InvalidMessage) {
+        if (obj) {
+            if (cmd.isEmpty()) {
+                show();
+            }
+            QDBusConnection::sessionBus().send(message.createReply());
+        } else {
+            QDBusConnection::sessionBus().send(message.createErrorReply(QDBusError::InvalidArgs, QString("not found url:") + url));
+        }
+    }
+}
+
 void DccManager::doShowPage(DccObject *obj, const QString &cmd)
 {
-    if (m_plugins->isDeleting() || !obj || (m_activeObject == obj && cmd.isEmpty()))
+    if (m_plugins->isDeleting() || !obj) {
         return;
-    // m_backwardBtn->setVisible(obj != m_root);
+    }
+    // 禁用首页
+    if (obj == m_root) {
+        if (m_root->getChildren().isEmpty()) {
+            return;
+        }
+        obj = m_root->getChildren().first();
+    }
+    if (m_activeObject == obj && cmd.isEmpty()) {
+        return;
+    }
+    if (!cmd.isEmpty()) {
+        Q_EMIT obj->active(cmd);
+        return;
+    }
     QList<DccObject *> modules;
     DccObject *triggeredObj = obj;
     if (triggeredObj->pageType() == DccObject::MenuEditor && !triggeredObj->getChildren().isEmpty()) {
@@ -456,31 +583,42 @@ void DccManager::updateModuleConfig(const QString &key)
     }
 }
 
-void DccManager::onTriggered()
-{
-    DccObject *obj = qobject_cast<DccObject *>(sender());
-    if (obj) {
-        showPage(obj, QString());
-    }
-}
-
 void DccManager::onVisible(bool visible)
 {
+    if (!m_root) {
+        return;
+    }
     DccObject *obj = qobject_cast<DccObject *>(sender());
     if (!obj) {
         return;
     }
     if (visible) {
+        QVector<DccObject *> objs;
+        objs.append(obj->getChildren());
+        while (!objs.isEmpty()) {
+            auto o = objs.takeFirst();
+            if (o->isVisibleToApp()) {
+                objs.append(o->getChildren());
+            } else {
+                connect(o, &DccObject::visibleToAppChanged, this, &DccManager::onVisible, Qt::ConnectionType(Qt::QueuedConnection | Qt::UniqueConnection));
+                removeObjectFromParent(o);
+                DccObject::Private::FromObject(m_hideObjects)->addChild(o, false);
+            }
+        }
         DccObject::Private::FromObject(m_hideObjects)->removeChild(obj);
         addObjectToParent(obj);
     } else {
         removeObjectFromParent(obj);
-        DccObject::Private::FromObject(m_hideObjects)->addChild(obj);
+        DccObject::Private::FromObject(m_hideObjects)->addChild(obj, false);
     }
 }
 
 void DccManager::onObjectAdded(DccObject *obj)
 {
+    if (!m_root) {
+        return;
+    }
+    m_searchModel->addSearchData(obj, QString(), QString());
     QVector<DccObject *> objs;
     objs.append(obj);
     while (!objs.isEmpty()) {
@@ -488,37 +626,41 @@ void DccManager::onObjectAdded(DccObject *obj)
         connect(o, &DccObject::childAdded, this, &DccManager::onObjectAdded);
         connect(o, &DccObject::childRemoved, this, &DccManager::onObjectRemoved);
         connect(o, &DccObject::displayNameChanged, this, &DccManager::onObjectDisplayChanged);
-        connect(o, &DccObject::triggered, this, &DccManager::onTriggered, Qt::QueuedConnection);
         connect(o, &DccObject::visibleToAppChanged, this, &DccManager::onVisible, Qt::ConnectionType(Qt::QueuedConnection | Qt::UniqueConnection));
-        m_searchModel->addSearchData(obj, QString(), QString());
         objs.append(o->getChildren());
     }
 }
 
 void DccManager::onObjectRemoved(DccObject *obj)
 {
+    if (!m_root) {
+        return;
+    }
     QVector<DccObject *> objs;
     objs.append(obj);
     while (!objs.isEmpty()) {
         auto o = objs.takeFirst();
         disconnect(o, &DccObject::childAdded, this, nullptr);
         disconnect(o, &DccObject::childRemoved, this, nullptr);
-        disconnect(o, &DccObject::triggered, this, nullptr);
         disconnect(o, &DccObject::displayNameChanged, this, nullptr);
         m_searchModel->removeSearchData(o, QString());
         objs.append(o->getChildren());
     }
-
+    DccObject *parentObj = m_root;
     for (auto &&o : m_currentObjects) {
         if (o == obj) {
-            showPage(QString());
+            doShowPage(parentObj, QString());
             break;
         }
+        parentObj = o;
     }
 }
 
 void DccManager::onObjectDisplayChanged()
 {
+    if (!m_root) {
+        return;
+    }
     DccObject *obj = qobject_cast<DccObject *>(sender());
     if (obj) {
         m_searchModel->removeSearchData(obj, QString());
@@ -528,7 +670,7 @@ void DccManager::onObjectDisplayChanged()
 
 bool DccManager::addObjectToParent(DccObject *obj)
 {
-    if (DccObject *parentObj = findObject(obj->parentName())) {
+    if (DccObject *parentObj = findParent(obj)) {
         DccObject::Private::FromObject(parentObj)->addChild(obj);
         return true;
     }
@@ -583,4 +725,44 @@ void DccManager::onQuit()
     // #endif
 }
 
+void DccManager::waitLoadFinished() const
+{
+    if (!m_plugins->loadFinished()) {
+        QEventLoop loop;
+        QTimer timer;
+        connect(&timer, &QTimer::timeout, &loop, &QEventLoop::quit);
+        connect(m_plugins, &PluginManager::loadAllFinished, &loop, &QEventLoop::quit);
+        timer.start(5000);
+        loop.exec();
+    }
+}
+
+void DccManager::doGetAllModule(const QDBusMessage message) const
+{
+    waitLoadFinished();
+    DccObject *root = m_root;
+    QList<QPair<DccObject *, QStringList>> modules;
+    for (auto &&child : root->getChildren()) {
+        modules.append({ child, { child->name(), child->displayName() } });
+    }
+
+    QJsonArray arr;
+    while (!modules.isEmpty()) {
+        const auto &urlInfo = modules.takeFirst();
+        QJsonObject obj;
+        obj.insert("url", urlInfo.second.at(0));
+        obj.insert("displayName", urlInfo.second.at(1));
+        obj.insert("weight", (int)(urlInfo.first->weight()));
+        arr.append(obj);
+        const QList<DccObject *> &children = urlInfo.first->getChildren();
+        for (auto it = children.crbegin(); it != children.crend(); ++it)
+            modules.prepend({ *it, { urlInfo.second.at(0) + "/" + (*it)->name(), urlInfo.second.at(1) + "/" + (*it)->displayName() } });
+    }
+
+    QJsonDocument doc;
+    doc.setArray(arr);
+    QString json = doc.toJson(QJsonDocument::Compact);
+    QDBusConnection::sessionBus().send(message.createReply(json));
+}
+
 } // namespace dccV25
