@@ -7,7 +7,10 @@
 #include "dccobject.h"
 
 #include <DConfig>
+#include <DSysInfo>
 
+#include <QDBusContext>
+#include <QDBusMessage>
 #include <QObject>
 
 QT_BEGIN_NAMESPACE
@@ -21,7 +24,7 @@ class NavigationModel;
 class SearchModel;
 class PluginManager;
 
-class DccManager : public DccApp
+class DccManager : public DccApp, protected QDBusContext
 {
     Q_OBJECT
 public:
@@ -41,7 +44,11 @@ public:
 
     inline DccObject *activeObject() const override { return m_activeObject; }
 
-    inline const QVector<DccObject *> &currentObjects() const { return m_currentObjects; }
+    Q_INVOKABLE inline const QVector<DccObject *> &currentObjects() const { return m_currentObjects; }
+
+    Q_INVOKABLE DccApp::UosEdition uosEdition() const;
+
+    Q_INVOKABLE bool isTreeland() const;
 
     inline const QSet<QString> &hideModule() const { return m_hideModule; }
 
@@ -51,34 +58,36 @@ public Q_SLOTS:
     void removeObject(DccObject *obj) override;
     void removeObject(const QString &name) override;
     void showPage(const QString &url) override;
+    void showPage(DccObject *obj);
     void showPage(DccObject *obj, const QString &cmd) override;
+    void toBack();
     QWindow *mainWindow() const override;
     QAbstractItemModel *navModel() const override;
     QSortFilterProxyModel *searchModel() const override;
-    void setShowPath(const QString &path) override;
+
+    void show();
     void showHelp();
     // DBus Search
     QString search(const QString &json);
     bool stop(const QString &json);
     bool action(const QString &json);
+    QString GetAllModule();
 
 Q_SIGNALS:
     void activeItemChanged(QQuickItem *item);
     void hideModuleChanged(const QSet<QString> &hideModule);
-    //     void pathChanged(const QString &path);
-    //     void rootChanged(DccObject *root);
-    //     void activeObjectChanged(DccObject *activeObject);
 
 private:
     void initConfig();
     bool contains(const QSet<QString> &urls, const DccObject *obj);
     bool isEqual(const QString &url, const DccObject *obj);
-    DccObject *findObject(const QString &url);
+    DccObject *findObject(const QString &url, bool onlyRoot = false);
+    DccObject *findParent(const DccObject *obj);
 
 private Q_SLOTS:
+    void waitShowPage(const QString &url, const QDBusMessage message);
     void doShowPage(DccObject *obj, const QString &cmd);
     void updateModuleConfig(const QString &key);
-    void onTriggered();
     void onVisible(bool visible);
     void onObjectAdded(DccObject *obj);
     void onObjectRemoved(DccObject *obj);
@@ -86,6 +95,8 @@ private Q_SLOTS:
     bool addObjectToParent(DccObject *obj);
     bool removeObjectFromParent(DccObject *obj);
     void onQuit();
+    void waitLoadFinished() const;
+    void doGetAllModule(const QDBusMessage message) const;
 
 private:
     DccObject *m_root;
