@@ -7,6 +7,8 @@
 #include "dccfactory.h"
 #include "dccmanager.h"
 
+#include <DIconTheme>
+
 #include <QDebug>
 #include <QDir>
 #include <QElapsedTimer>
@@ -40,7 +42,7 @@ enum PluginStatus {
     ModuleErr = 0x00800000,
     // data 0x0000FF00
     DataBegin = 0x00000100,
-    DataLoad = 0x00000300,
+    DataLoad = 0x00000200,
     DataEnd = 0x00004000,
     DataErr = 0x00008000,
     // mainObj 0x000000FF
@@ -62,6 +64,7 @@ struct PluginData
     DccObject *mainObj;
     DccObject *soObj;
     QObject *data;
+    QThread *thread;
     uint status;
 
     PluginData(const QString &_name, const QString &_path)
@@ -71,6 +74,7 @@ struct PluginData
         , mainObj(nullptr)
         , soObj(nullptr)
         , data(nullptr)
+        , thread(nullptr)
         , status(PluginBegin)
     {
     }
@@ -88,6 +92,7 @@ public:
 
 protected:
     void run() override;
+    void doRun();
 
 protected:
     PluginManager *m_pManager;
@@ -95,6 +100,13 @@ protected:
 };
 
 void LoadPluginTask::run()
+{
+    m_data->thread = QThread::currentThread();
+    doRun();
+    m_data->thread = nullptr;
+}
+
+void LoadPluginTask::doRun()
 {
     m_pManager->updatePluginStatus(m_data, DataBegin, "load plugin begin");
     // {main.qml}
@@ -131,7 +143,7 @@ void LoadPluginTask::run()
                 DccFactory *factory = qobject_cast<DccFactory *>(loader.instance());
                 if (!factory) {
                     m_pManager->updatePluginStatus(m_data, DataErr, "The plugin isn't a DccFactory." + soPath);
-                    delete loader.instance();
+                    loader.unload();
                     break;
                 }
                 dataObj = factory->create();
@@ -142,7 +154,6 @@ void LoadPluginTask::run()
                 if (soObj && soObj->parent()) {
                     soObj->setParent(nullptr);
                 }
-                delete factory;
             } while (false);
         }
     } else {
@@ -237,9 +248,9 @@ void PluginManager::loadPlugin(PluginData *plugin)
     } else if (plugin->status & MainObjEnd) {
         addMainObject(plugin);
         updatePluginStatus(plugin, PluginEnd);
-    } else if (plugin->status & DataEnd) {
+    } else if ((plugin->status & (DataEnd | MainObjLoad)) == DataEnd) {
         loadMain(plugin);
-    } else if (plugin->status & ModuleEnd) {
+    } else if ((plugin->status & (ModuleEnd | DataBegin)) == ModuleEnd) {
         if (plugin->module) {
             disconnect(plugin->module, nullptr, this, nullptr);
             if (plugin->module->isVisibleToApp()) {
@@ -251,7 +262,7 @@ void PluginManager::loadPlugin(PluginData *plugin)
         } else {
             threadPool()->start(new LoadPluginTask(plugin, this));
         }
-    } else if (plugin->status & MetaDataEnd) {
+    } else if ((plugin->status & (MetaDataEnd | ModuleLoad)) == MetaDataEnd) {
         DccManager::installTranslator(plugin->name);
         loadModule(plugin);
     } else {
@@ -287,6 +298,7 @@ void PluginManager::loadMetaData(PluginData *plugin)
         return;
     }
     // metadata
+#if 0 // 文件夹有版本信息，不需要用metadata.json判断
     const QString metadataPath = plugin->path + "/metadata.json";
     QFile metadataFile(metadataPath);
     if (!metadataFile.open(QIODevice::ReadOnly)) {
@@ -304,6 +316,7 @@ void PluginManager::loadMetaData(PluginData *plugin)
         updatePluginStatus(plugin, MetaDataErr | PluginEnd, "plugin's version is too low:" + metaData.value("Version").toString());
         return;
     }
+#endif
     updatePluginStatus(plugin, MetaDataEnd);
 }
 
@@ -313,7 +326,7 @@ void PluginManager::loadModule(PluginData *plugin)
         return;
     }
     const QString qmlPath = plugin->path + "/" + plugin->name + ".qml";
-    updatePluginStatus(plugin, ModuleLoad, ": load module" + qmlPath);
+    updatePluginStatus(plugin, ModuleLoad, ": load module " + qmlPath);
     if (QFile::exists(qmlPath)) {
         QQmlComponent *component = new QQmlComponent(m_manager->engine());
         component->setProperty("PluginData", QVariant::fromValue(plugin));
@@ -332,9 +345,15 @@ void PluginManager::loadMain(PluginData *plugin)
     if (isDeleting()) {
         return;
     }
-    const QString qmlPath = plugin->path + "/main.qml";
-    updatePluginStatus(plugin, MainObjLoad, "load main");
-    if (QFile::exists(qmlPath)) {
+    updatePluginStatus(plugin, MainObjLoad, "load Main");
+    QString qmlPath = plugin->path + "/" + plugin->name + "Main.qml";
+    if (!QFile::exists(qmlPath)) {
+        qmlPath = plugin->path + "/main.qml";
+        if (!QFile::exists(qmlPath)) {
+            qmlPath.clear();
+        }
+    }
+    if (!qmlPath.isEmpty()) {
         QQmlComponent *component = new QQmlComponent(m_manager->engine());
         component->setProperty("PluginData", QVariant::fromValue(plugin));
         component->loadUrl(qmlPath, QQmlComponent::Asynchronous);
@@ -344,7 +363,7 @@ void PluginManager::loadMain(PluginData *plugin)
             createMain(component);
         }
     } else {
-        updatePluginStatus(plugin, MainObjErr | MainObjEnd, "main.qml not exists");
+        updatePluginStatus(plugin, MainObjErr | MainObjEnd, "Main.qml not exists");
     }
 }
 
@@ -380,7 +399,7 @@ void PluginManager::createMain(QQmlComponent *component)
         updatePluginStatus(plugin, MainObjErr | MainObjEnd, " component create main object error:" + component->errorString());
     } else {
         QQmlContext *context = new QQmlContext(component->engine(), component);
-        context->setContextProperties({ { "dccData", QVariant::fromValue(plugin->data) } });
+        context->setContextProperties({ { "dccData", QVariant::fromValue(plugin->data) }, { "dccModule", QVariant::fromValue(plugin->module) } });
         QObject *object = component->create(context);
         // component->createWithInitialProperties({}, context);
         if (!object) {
@@ -416,6 +435,8 @@ void PluginManager::addMainObject(PluginData *plugin)
                 connect(plugin->mainObj, &DccObject::iconChanged, plugin->module, &DccObject::setIcon);
                 connect(plugin->mainObj, &DccObject::badgeChanged, plugin->module, &DccObject::setBadge);
                 connect(plugin->mainObj, &DccObject::visibleChanged, plugin->module, &DccObject::setVisible);
+                connect(plugin->mainObj, &DccObject::active, plugin->module, &DccObject::active);
+                connect(plugin->mainObj, &DccObject::deactive, plugin->module, &DccObject::deactive);
             }
         } else {
         }
@@ -449,7 +470,7 @@ void PluginManager::mainLoading()
 void PluginManager::onHideModuleChanged(const QSet<QString> &hideModule)
 {
     for (auto &&plugin : m_plugins) {
-        if ((plugin->status & PluginEnd) && ((!(plugin->status & (MetaDataEnd | MetaDataErr))) && (!hideModule.contains(plugin->name)))) {
+        if ((plugin->status & PluginEnd) && (((plugin->status & (MetaDataEnd | MetaDataErr | ModuleLoad)) == MetaDataEnd) && (!hideModule.contains(plugin->name)))) {
             // 加载完成，没检查MetaData也没错误，不在hideModule中，则需要重新加载
             plugin->status &= ~PluginEnd;
             loadPlugin(plugin);
@@ -489,21 +510,33 @@ void PluginManager::loadModules(DccObject *root, bool async, const QStringList &
             pluginList += plugindir.entryInfoList(QDir::Dirs | QDir::NoDotAndDotDot);
         }
     }
-
+    QStringList paths = Dtk::Gui::DIconTheme::dciThemeSearchPaths();
     for (auto &lib : pluginList) {
         const QString &filepath = lib.absoluteFilePath();
         auto filename = lib.fileName();
         PluginData *plugin = new PluginData(lib.baseName(), filepath);
         m_plugins.append(plugin);
         loadPlugin(plugin);
+        paths.prepend(filepath);
     }
+    Dtk::Gui::DIconTheme::setDciThemeSearchPaths(paths);
 }
 
 void PluginManager::cancelLoad()
 {
     if (m_threadPool) {
-        m_threadPool->waitForDone(500);
         m_threadPool->clear();
+        for (auto &&plugin : m_plugins) {
+            if (plugin->thread && plugin->thread->isRunning()) {
+                plugin->thread->wait(50);
+                if (plugin->thread->isRunning()) {
+                    qCWarning(dccLog()) << plugin->name << ": status" << QString::number(plugin->status, 16) << plugin->thread << "thread exit timeout";
+                    plugin->thread->terminate();
+                    plugin->thread->wait(50);
+                    plugin->thread = nullptr;
+                }
+            }
+        }
         delete m_threadPool;
         m_threadPool = nullptr;
     }
