@@ -192,9 +192,7 @@ CommonInfoWork::CommonInfoWork(CommonInfoModel *model, QObject *parent)
         m_commomModel->themeEnabledChanged(m_commomModel->themeEnabled());
     });
 
-    connect(m_commonInfoProxy, &CommonInfoProxy::resetGrubEditAuthEnabled, this, [=](){
-        m_commomModel->grubEditAuthEnabledChanged(m_commomModel->grubEditAuthEnabled());
-    });
+    connect(m_commonInfoProxy, &CommonInfoProxy::resetGrubEditAuthEnabled, this, &CommonInfoWork::resetEditAuthEnabled);
 }
 
 CommonInfoWork::~CommonInfoWork()
@@ -452,6 +450,11 @@ void CommonInfoWork::exportMessage(QString filePath)
     file.close();
 }
 
+void CommonInfoWork::resetEditAuthEnabled()
+{
+    Q_EMIT m_commomModel->grubEditAuthEnabledChanged(m_commomModel->grubEditAuthEnabled());
+}
+
 void CommonInfoWork::setBootDelay(bool value)
 {
     qDebug()<<" CommonInfoWork::setBootDelay  value =  "<< value;
@@ -555,6 +558,7 @@ void CommonInfoWork::setEnableDeveloperMode(bool enabled)
     contentFile->write(content.toLocal8Bit());
     contentFile->close();
 
+    m_commomModel->setNeedShowModalDialog(true);
     auto pathType = "-c";
     QStringList sl;
     sl << "zh_CN" << "zh_TW";
@@ -570,6 +574,7 @@ void CommonInfoWork::setEnableDeveloperMode(bool enabled)
         } else {
             qInfo() << QString("On %1, Remove developer mode Disclaimer!").arg(current_date);
         }
+        m_commomModel->setNeedShowModalDialog(false);
         contentFile->remove();
         contentFile->deleteLater();
         m_process->deleteLater();
