@@ -20,6 +20,7 @@
 #include <QTimer>
 
 #include <DConfig>
+#include <DDBusSender>
 
 DCORE_USE_NAMESPACE
 Q_LOGGING_CATEGORY(DdcPersonalWorker, "dcc-personal-workder")
@@ -28,44 +29,22 @@ static const std::vector<int> OPACITY_SLIDER{ 0, 25, 40, 55, 70, 85, 100 };
 
 const QList<int> FontSizeList{ 11, 12, 13, 14, 15, 16, 18, 20 };
 
-const QString ORG_KDE_KWIN_DECORATION = QStringLiteral("org.kde.kwin.decoration");
-const QString ORG_KDE_KWIN_DECORATION_TITLEBAR = QStringLiteral("org.kde.kwin.decoration.titlebar");
-const QString ORG_KDE_KWIN_COMPOSITING = QStringLiteral("org.kde.kwin.compositing");
-const QString ORG_DEEPIN_CONTROL_CENTER = QStringLiteral("org.deepin.dde.control-center");
-const QString CONTROL_CENTER_PERSONALIZATION = QStringLiteral("org.deepin.dde.control-center.personalization");
-const QString ORG_KDE_KWIN = QStringLiteral("org.kde.kwin");
-
-const QString SCROLLBAR_POLICY_KEY = QStringLiteral("scrollbarPolicyStatus");
-const QString COMPACT_MODE_DISPLAY_KEY = QStringLiteral("compactDisplayStatus");
-const QString WINDOW_EFFECT_TYPE_KEY = QStringLiteral("user_type");
-const QString TITLE_BAR_DEFAULT_HEIGHT_KEY = QStringLiteral("defaultTitlebarHeight");
-const QString TITLE_BAR_HEIGHT_SUPPORT_COMPACT_DISPLAY = QStringLiteral("titleBarHeightSupportCompactDisplay");
-const QString TITLE_BAR_HEIGHT_KEY = QStringLiteral("titlebarHeight");
-const QString EffectMoveWindowArg = "kwin4_effect_translucency";
-
-const int RENDER_DPI = 72;
-const double DPI = 96;
-
-double ptToPx(double pt)
-{
-    double px = pt / RENDER_DPI * DPI + 0.5;
-    return px;
-}
-
-double pxToPt(double px)
-{
-    double pt = px * RENDER_DPI / DPI;
-    return pt;
-}
+constexpr auto ORG_DEEPIN_CONTROL_CENTER = "org.deepin.dde.control-center";
+constexpr auto CONTROL_CENTER_PERSONALIZATION = "org.deepin.dde.control-center.personalization";
+constexpr auto DTK_PREFERENCE_NAME = "org.deepin.dtk.preference";
+constexpr auto SCROLLBAR_POLICY_CONFIG_KEY = "scrollbarPolicyStatus";
+constexpr auto COMPACT_MODE_DISPLAY_KEY = "compactDisplayStatus";
+constexpr auto TITLE_BAR_HEIGHT_SUPPORT_COMPACT_DISPLAY = "titleBarHeightSupportCompactDisplay";
+constexpr auto SIZE_MODE_KEY = "sizeMode";
+constexpr auto SCROLLBAR_POLICY_KEY = "scrollBarPolicy";
 
 PersonalizationWorker::PersonalizationWorker(PersonalizationModel *model, QObject *parent)
     : QObject(parent)
     , m_model(model)
     , m_personalizationDBusProxy(new PersonalizationDBusProxy(this))
     , m_wallpaperWorker(new WallpaperWorker(m_personalizationDBusProxy, m_model->getWallpaperModel(), this))
-    , m_kwinTitleBarConfig(DConfig::create(ORG_KDE_KWIN_DECORATION, ORG_KDE_KWIN_DECORATION_TITLEBAR, "", this))
-    , m_kwinCompositingConfig(DConfig::create(ORG_KDE_KWIN, ORG_KDE_KWIN_COMPOSITING, "", this))
     , m_personalizationConfig(DConfig::create(ORG_DEEPIN_CONTROL_CENTER, CONTROL_CENTER_PERSONALIZATION, "", this))
+    , m_dtkConfig(DConfig::createGeneric(DTK_PREFERENCE_NAME, "", this))
 {
     ThemeModel *cursorTheme = m_model->getMouseModel();
     ThemeModel *windowTheme = m_model->getWindowModel();
@@ -83,15 +62,12 @@ PersonalizationWorker::PersonalizationWorker(PersonalizationModel *model, QObjec
     connect(m_personalizationDBusProxy, &PersonalizationDBusProxy::FontSizeChanged, this, &PersonalizationWorker::FontSizeChanged);
     connect(m_personalizationDBusProxy, &PersonalizationDBusProxy::Refreshed, this, &PersonalizationWorker::onRefreshedChanged);
 
-    connect(m_personalizationDBusProxy, &PersonalizationDBusProxy::WMChanged, this, &PersonalizationWorker::onToggleWM);
     connect(m_personalizationDBusProxy, &PersonalizationDBusProxy::OpacityChanged, this, &PersonalizationWorker::refreshOpacity);
     connect(m_personalizationDBusProxy, &PersonalizationDBusProxy::QtActiveColorChanged, this, &PersonalizationWorker::refreshActiveColor);
-    connect(m_personalizationDBusProxy, &PersonalizationDBusProxy::compositingAllowSwitchChanged, this, &PersonalizationWorker::onCompositingAllowSwitch);
-    connect(m_personalizationDBusProxy, &PersonalizationDBusProxy::compositingEnabledChanged, this, &PersonalizationWorker::onWindowWM);
     connect(m_personalizationDBusProxy, &PersonalizationDBusProxy::WindowRadiusChanged, this, &PersonalizationWorker::onWindowRadiusChanged);
-    connect(m_personalizationDBusProxy, &PersonalizationDBusProxy::DTKSizeModeChanged, this, &PersonalizationWorker::onCompactDisplayChanged);
-    connect(m_personalizationDBusProxy, &PersonalizationDBusProxy::scrollBarPolicyChanged, this, &PersonalizationWorker::setScrollBarPolicy);
     connect(m_personalizationDBusProxy, &PersonalizationDBusProxy::WallpaperURlsChanged, this, &PersonalizationWorker::onWallpaperUrlsChanged);
+    connect(qApp, &QGuiApplication::screenAdded, this, &PersonalizationWorker::onScreensChanged);
+    connect(qApp, &QGuiApplication::screenRemoved, this, &PersonalizationWorker::onScreensChanged);
     connect(m_personalizationDBusProxy, &PersonalizationDBusProxy::Changed, this, [this](const QString &propertyName, const QString &value) {
         qCDebug(DdcPersonalWorker) << "ChangeProperty is " << propertyName << "; value is" << value;
         if (propertyName == "globaltheme") {
@@ -99,11 +75,8 @@ PersonalizationWorker::PersonalizationWorker(PersonalizationModel *model, QObjec
         }
     });
 
-    connect(m_kwinTitleBarConfig, &DConfig::valueChanged, this, &PersonalizationWorker::onKWinTitleBarConfigChanged);
-    connect(m_kwinCompositingConfig, &DConfig::valueChanged, this, &PersonalizationWorker::onKWinCompositingConfigChanged);
     connect(m_personalizationConfig, &DConfig::valueChanged, this, &PersonalizationWorker::onPersonalizationConfigChanged);
-
-    m_personalizationDBusProxy->isEffectLoaded("magiclamp", this, SLOT(onMiniEffectChanged(bool)));
+    connect(m_dtkConfig, &DConfig::valueChanged, this, &PersonalizationWorker::onDTKConfigChanged);
 
     m_themeModels["gtk"] = windowTheme;
     m_themeModels["icon"] = iconTheme;
@@ -112,18 +85,17 @@ PersonalizationWorker::PersonalizationWorker(PersonalizationModel *model, QObjec
     m_fontModels["standardfont"] = fontStand;
     m_fontModels["monospacefont"] = fontMono;
 
-    m_wallpaperWorker->fecthData();
+    m_wallpaperWorker->fetchData();
 }
 
 void PersonalizationWorker::active()
 {
     m_personalizationDBusProxy->blockSignals(false);
 
-    refreshWMState();
     refreshOpacity(m_personalizationDBusProxy->opacity());
     refreshActiveColor(m_personalizationDBusProxy->qtActiveColor());
-    onCompositingAllowSwitch(m_personalizationDBusProxy->compositingAllowSwitch());
-    onWallpaperUrlsChanged({});
+    onWallpaperUrlsChanged();
+    onScreensChanged();
 
     m_model->setCurrentSelectScreen(qApp->primaryScreen()->name());
     m_model->getWindowModel()->setDefault(m_personalizationDBusProxy->gtkTheme());
@@ -135,15 +107,11 @@ void PersonalizationWorker::active()
     m_model->setWindowRadius(m_personalizationDBusProxy->windowRadius());
     m_model->getFontSizeModel()->setFontSize(ptToPx(m_personalizationDBusProxy->fontSize()));
     m_model->setCompactDisplay(m_personalizationDBusProxy->getDTKSizeMode());
-    m_model->setScrollBarPolicy(m_personalizationDBusProxy->getScrollBarPolicy());
 
-    int titleBarHight = m_kwinTitleBarConfig->value(TITLE_BAR_HEIGHT_KEY).toInt();
-    m_model->setTitleBarHeight(titleBarHight);
-    int titleBarDefHight = m_kwinTitleBarConfig->value(TITLE_BAR_DEFAULT_HEIGHT_KEY).toInt();
-    m_model->setTitleBarDefaultHeight(titleBarDefHight);
-    int windowEffectType = m_kwinCompositingConfig->value(WINDOW_EFFECT_TYPE_KEY).toInt();
-    m_model->setWindowEffectType(windowEffectType);
-    QString scrollbarConfig = m_personalizationConfig->value(SCROLLBAR_POLICY_KEY).toString();
+    m_model->setScrollBarPolicy(m_dtkConfig->value(SCROLLBAR_POLICY_KEY).toInt());
+    m_model->setCompactDisplay(m_dtkConfig->value(SIZE_MODE_KEY).toInt());
+
+    QString scrollbarConfig = m_personalizationConfig->value(SCROLLBAR_POLICY_CONFIG_KEY).toString();
     m_model->setScrollBarPolicyConfig(scrollbarConfig);
     QString compactDisplayConfig = m_personalizationConfig->value(COMPACT_MODE_DISPLAY_KEY).toString();
     m_model->setCompactDisplayConfig(compactDisplayConfig);
@@ -200,12 +168,6 @@ void PersonalizationWorker::addList(ThemeModel *model, const QString &type, cons
     }
 }
 
-void PersonalizationWorker::refreshWMState()
-{
-    m_personalizationDBusProxy->CurrentWM(this, SLOT(onToggleWM(const QString &)));
-    m_model->setIsMoveWindow(m_personalizationDBusProxy->isEffectLoaded(EffectMoveWindowArg));
-}
-
 void PersonalizationWorker::FontSizeChanged(const double value) const
 {
     FontSizeModel *fontSizeModel = m_model->getFontSizeModel();
@@ -240,33 +202,11 @@ void PersonalizationWorker::onRefreshedChanged(const QString &type)
     }
 }
 
-void PersonalizationWorker::onToggleWM(const QString &wm)
-{
-    qCDebug(DdcPersonalWorker) << "onToggleWM: " << wm;
-    m_model->setIs3DWm(wm == "deepin wm");
-}
-
-void PersonalizationWorker::onWindowWM(bool value)
-{
-    qDebug() << "onWindowWM: " << value;
-    m_model->setIs3DWm(value);
-}
-
-void PersonalizationWorker::onMiniEffectChanged(bool value)
-{
-    m_model->setMiniEffect(value ? 1 : 0);
-}
-
 void PersonalizationWorker::onWindowRadiusChanged(int value)
 {
     m_model->setWindowRadius(value);
 }
 
-void PersonalizationWorker::onCompositingAllowSwitch(bool value)
-{
-    m_model->setCompositingAllowSwitch(value);
-}
-
 void PersonalizationWorker::onCompactDisplayChanged(int value)
 {
     m_model->setCompactDisplay(value);
@@ -277,10 +217,18 @@ void PersonalizationWorker::onWindowEffectChanged(int value)
     m_model->setWindowEffectType(value);
 }
 
-void PersonalizationWorker::onWallpaperUrlsChanged(const QString &value)
+void PersonalizationWorker::onScreensChanged()
+{
+    QStringList screenNameList{};
+    for (const auto &screen : qApp->screens()) {
+        screenNameList << screen->name();
+    } 
+    m_model->setScreens(screenNameList);
+}
+
+void PersonalizationWorker::onWallpaperUrlsChanged()
 {
     // wallpaperUrls 存储着每个工作区和每个屏幕的壁纸, 若其改变, 需要刷新当前屏幕壁纸
-    Q_UNUSED(value)
     QVariantMap wallpaperMap;
     for (auto &screen : qApp->screens()) {
         QString url = m_personalizationDBusProxy->getCurrentWorkSpaceBackgroundForMonitor(screen->name());
@@ -308,9 +256,9 @@ void PersonalizationWorker::setFontList(FontModel *model, const QString &type, c
     m_personalizationDBusProxy->Show(type, l, watcher, SLOT(onShow(const QString &)));
 }
 
-void PersonalizationWorker::setTitleBarHeight(int value)
+void PersonalizationWorker::setTitleBarHeight(int)
 {
-    m_kwinTitleBarConfig->setValue(TITLE_BAR_HEIGHT_KEY, value);
+
 }
 
 void PersonalizationWorker::setDiabledCompactToTitleHeight()
@@ -355,11 +303,6 @@ void PersonalizationWorker::refreshActiveColor(const QString &color)
     m_model->setActiveColor(color);
 }
 
-bool PersonalizationWorker::allowSwitchWM()
-{
-    return m_personalizationDBusProxy->AllowSwitch();
-}
-
 void PersonalizationWorker::refreshOpacity(double opacity)
 {
     qCDebug(DdcPersonalWorker) << QString("opacity: %1").arg(opacity);
@@ -388,42 +331,14 @@ void PersonalizationWorker::setFontSize(const int value)
     m_personalizationDBusProxy->setFontSize(pxToPt(value));
 }
 
-void PersonalizationWorker::switchWM()
+void PersonalizationWorker::setWindowEffect(int)
 {
-    //check is allowed to switch wm
-    bool allow = allowSwitchWM();
-    if (!allow)
-        return;
 
-    m_personalizationDBusProxy->RequestSwitchWM();
 }
 
-void PersonalizationWorker::windowSwitchWM(bool value)
+void PersonalizationWorker::setMovedWindowOpacity(bool)
 {
-    m_personalizationDBusProxy->setCompositingEnabled(value);
-}
 
-void PersonalizationWorker::setWindowEffect(int value)
-{
-    qCWarning(DdcPersonalWorker) << "windowSwitchWM switch to: " << value;
-    m_kwinCompositingConfig->setValue(WINDOW_EFFECT_TYPE_KEY, value);
-}
-
-void PersonalizationWorker::movedWindowSwitchWM(bool value)
-{
-    if (value) {
-        m_personalizationDBusProxy->loadEffect(EffectMoveWindowArg);
-    } else {
-        m_personalizationDBusProxy->unloadEffect(EffectMoveWindowArg);
-    }
-
-    //设置kwin接口后, 等待50ms给kwin反应，根据isEffectLoaded返回值确定真实状态
-    QTimer::singleShot(50, [this] {
-        bool isLoaded =  m_personalizationDBusProxy->isEffectLoaded(EffectMoveWindowArg);
-        qCDebug(DdcPersonalWorker) << "Moved window switch WM, load effect translucency: " << isLoaded;
-        m_model->setIsMoveWindow(isLoaded);
-        // m_model->setIsMoveWindowDconfig(value);
-    });
 }
 
 void PersonalizationWorker::setOpacity(int opacity)
@@ -431,22 +346,9 @@ void PersonalizationWorker::setOpacity(int opacity)
     m_personalizationDBusProxy->setOpacity(sliderValutToOpacity(opacity));
 }
 
-void PersonalizationWorker::setMiniEffect(int effect)
+void PersonalizationWorker::setMiniEffect(int)
 {
-    switch (effect) {
-    case 0:
-        qCDebug(DdcPersonalWorker) << "scale";
-        m_personalizationDBusProxy->unloadEffect("magiclamp");
-        m_model->setMiniEffect(effect);
-        break;
-    case 1:
-        qCDebug(DdcPersonalWorker) << "magiclamp";
-        m_personalizationDBusProxy->loadEffect("magiclamp");
-        m_model->setMiniEffect(effect);
-        break;
-    default:
-        break;
-    }
+
 }
 
 void PersonalizationWorker::setActiveColor(const QString &hexColor)
@@ -454,6 +356,11 @@ void PersonalizationWorker::setActiveColor(const QString &hexColor)
     m_personalizationDBusProxy->setQtActiveColor(hexColor);
 }
 
+void PersonalizationWorker::setActiveColors(const QString &activeColors)
+{
+    m_personalizationDBusProxy->setActiveColors(activeColors);
+}
+
 void PersonalizationWorker::setWindowRadius(int radius)
 {
     m_personalizationDBusProxy->setWindowRadius(radius);
@@ -473,15 +380,28 @@ void PersonalizationWorker::setCompactDisplay(bool value)
         } 
     }
 
+    m_dtkConfig->setValue(SIZE_MODE_KEY, value);
+    // TODO delete
     m_personalizationDBusProxy->setDTKSizeMode(int(value));
 }
 
 
 void PersonalizationWorker::setScrollBarPolicy(int policy)
 {
+    m_dtkConfig->setValue(SCROLLBAR_POLICY_KEY, policy);
+    // TODO delete
     m_personalizationDBusProxy->setScrollBarPolicy( policy);
 }
 
+void PersonalizationWorker::goDownloadTheme()
+{
+    DDBusSender().interface("com.home.appstore.client")
+    .path("/com/home/appstore/client")
+    .service("com.home.appstore.client")
+    .method("openBusinessUri")
+    .arg(QString("searchApp?keyword=theme")).call();
+}
+
 template<typename T>
 T PersonalizationWorker::toSliderValue(std::vector<T> list, T value)
 {
@@ -494,28 +414,9 @@ T PersonalizationWorker::toSliderValue(std::vector<T> list, T value)
     return list.end() - list.begin();
 }
 
-void PersonalizationWorker::onKWinTitleBarConfigChanged(const QString &key)
-{
-    if (key == TITLE_BAR_HEIGHT_KEY) {
-        int value = m_kwinTitleBarConfig->value(key).toInt();
-        m_model->setTitleBarHeight(value);
-    } else if (key == WINDOW_EFFECT_TYPE_KEY) {
-        int value = m_kwinTitleBarConfig->value(key).toInt();
-        m_model->setWindowEffectType(value);
-    }
-}
-
-void PersonalizationWorker::onKWinCompositingConfigChanged(const QString &key)
-{
-    if (key == WINDOW_EFFECT_TYPE_KEY) {
-        int windowEffectType = m_kwinCompositingConfig->value(WINDOW_EFFECT_TYPE_KEY).toInt();
-        m_model->setWindowEffectType(windowEffectType);
-    }
-}
-
 void PersonalizationWorker::onPersonalizationConfigChanged(const QString &key)
 {
-    if (key == SCROLLBAR_POLICY_KEY) {
+    if (key == SCROLLBAR_POLICY_CONFIG_KEY) {
         QString value = m_personalizationConfig->value(key).toString();
         m_model->setScrollBarPolicyConfig(value);
     } else if (key == COMPACT_MODE_DISPLAY_KEY) {
@@ -524,6 +425,16 @@ void PersonalizationWorker::onPersonalizationConfigChanged(const QString &key)
     }
 }
 
+void PersonalizationWorker::onDTKConfigChanged(const QString &key)
+{
+    qWarning() << "PersonalizationWorker::onDTKConfigChanged" << key << m_dtkConfig->value(key);
+    if (key == SIZE_MODE_KEY) {
+        m_model->setCompactDisplay(m_dtkConfig->value(key).toBool());
+    } else if (key == SCROLLBAR_POLICY_KEY) {
+        m_model->setScrollBarPolicy(m_dtkConfig->value(key).toInt());
+    }
+}
+
 void PersonalizationWorker::setGlobalTheme(const QString &themeId)
 {
     qDebug() << "applied global theme" << themeId;
@@ -567,8 +478,9 @@ void PersonalizationWorker::setCursorTheme(const QString &id)
     }
 }
 
-void PersonalizationWorker::setBackgroundForMonitor(const QString &screenName, const QString &url)
+void PersonalizationWorker::setBackgroundForMonitor(const QString &screenName, const QString &url, bool isDark)
 {
+    Q_UNUSED(isDark)
     qInfo() << "Appearance SetMonitorBackground " << screenName << url;
     if (screenName.isEmpty() || url.isEmpty())
         return;
@@ -576,9 +488,11 @@ void PersonalizationWorker::setBackgroundForMonitor(const QString &screenName, c
     m_personalizationDBusProxy->SetCurrentWorkspaceBackgroundForMonitor(url, screenName);
 }
 
-QString PersonalizationWorker::getBackgroundForMonitor(const QString &screenName)
+void PersonalizationWorker::setLockBackForMonitor(const QString &screenName, const QString &url, bool isDark)
 {
-    return m_personalizationDBusProxy->getCurrentWorkSpaceBackgroundForMonitor(screenName);
+    Q_UNUSED(screenName)
+    Q_UNUSED(url)
+    Q_UNUSED(isDark)
 }
 
 PersonalizationWatcher::PersonalizationWatcher(PersonalizationWorker *work)
