@@ -23,36 +23,37 @@ class PersonalizationWorker : public QObject
     Q_OBJECT
 public:
     PersonalizationWorker(PersonalizationModel *model, QObject *parent = nullptr);
-    void active();
+    virtual void active();
     void deactive();
     void onGetList();
     void refreshTheme();
     void refreshFont();
 
 public Q_SLOTS:
-    void setDefaultByType(const QString &type, const QString &value);
-    void setDefault(const QJsonObject &value);
-    void setFontSize(const int value);
-    void switchWM();
-    void windowSwitchWM(bool value);
-    void movedWindowSwitchWM(bool value);
-    void setOpacity(int opcaity);
-    void setWindowEffect(int value);
-    void setMiniEffect(int effect);
-    void setActiveColor(const QString &hexColor);
-    void setWindowRadius(int radius);
-    void setCompactDisplay(bool value);
-    void setScrollBarPolicy(int policy);
-    void setTitleBarHeight(int value);
     void setDiabledCompactToTitleHeight();
-    void setGlobalTheme(const QString &themeId);
-    void setAppearanceTheme(const QString &id);
-    void setIconTheme(const QString &id);
-    void setCursorTheme(const QString &id);
+    void setScrollBarPolicy(int policy);
+    void setCompactDisplay(bool value);
+    void goDownloadTheme();
 
-    // wallpaper
-    void setBackgroundForMonitor(const QString &screenName, const QString &url);
-    QString getBackgroundForMonitor(const QString &screenName);
+    // 设置给Appearance分别在深色和浅色下的活动色
+    void setActiveColors(const QString &activeColors);
+
+    virtual void setDefaultByType(const QString &type, const QString &value);
+    virtual void setDefault(const QJsonObject &value);
+    virtual void setFontSize(const int value);
+    virtual void setOpacity(int opcaity);
+    virtual void setWindowEffect(int value);
+    virtual void setMiniEffect(int effect);
+    virtual void setMovedWindowOpacity(bool value);
+    virtual void setActiveColor(const QString &hexColor);
+    virtual void setWindowRadius(int radius);
+    virtual void setTitleBarHeight(int value);
+    virtual void setGlobalTheme(const QString &themeId);
+    virtual void setAppearanceTheme(const QString &id);
+    virtual void setIconTheme(const QString &id);
+    virtual void setCursorTheme(const QString &id);
+    virtual void setBackgroundForMonitor(const QString &screenName, const QString &url, bool isDark);
+    virtual void setLockBackForMonitor(const QString &screenName, const QString &url, bool isDark);
 
 signals:
     void personalizationChanged(const QString &propertyName, const QString &value);
@@ -62,42 +63,38 @@ private Q_SLOTS:
     void onGetFontFinished(const QString &category, const QString &json);
     void onGetThemeFinished(const QString &category, const QString &json);
     void onGetPicFinished(const QString &category, const QString &id, const QString &json);
-    //    void onGetActiveColorFinished(QDBusPendingCallWatcher *w);
     void onRefreshedChanged(const QString &type);
-    void onToggleWM(const QString &wm);
     void setFontList(FontModel *model, const QString &type, const QString &list);
-    void onCompositingAllowSwitch(bool value);
-    void onWindowWM(bool value);
-    void onMiniEffectChanged(bool value);
     void onWindowRadiusChanged(int value);
     void onCompactDisplayChanged(int value);
     void onWindowEffectChanged(int value);
-    void onWallpaperUrlsChanged(const QString &value);
+    void onScreensChanged();
+
+protected:
+    virtual void onWallpaperUrlsChanged();
 
 private:
     double sliderValutToOpacity(const int value) const;
     QList<QJsonObject> converToList(const QString &type, const QJsonArray &array);
     void addList(ThemeModel *model, const QString &type, const QJsonArray &array);
-    void refreshWMState();
     void refreshThemeByType(const QString &type);
     void refreshFontByType(const QString &type);
     void refreshOpacity(double opacity);
     void refreshActiveColor(const QString &color);
-    bool allowSwitchWM();
-    void onKWinTitleBarConfigChanged(const QString &key);
-    void onKWinCompositingConfigChanged(const QString &key);
     void onPersonalizationConfigChanged(const QString &key);
+    void onDTKConfigChanged(const QString &key);
 
     template<typename T>
     T toSliderValue(std::vector<T> list, T value);
 
-private:
+protected:
     PersonalizationModel *m_model;
     PersonalizationDBusProxy *m_personalizationDBusProxy;
+
+private:
     WallpaperWorker *m_wallpaperWorker;
-    Dtk::Core::DConfig *m_kwinTitleBarConfig;
-    Dtk::Core::DConfig *m_kwinCompositingConfig;
     Dtk::Core::DConfig *m_personalizationConfig;
+    Dtk::Core::DConfig *m_dtkConfig;
 
     QMap<QString, ThemeModel *> m_themeModels;
     QMap<QString, FontModel *> m_fontModels;
