@@ -6,6 +6,7 @@
 SoundEffectsData::SoundEffectsData()
     : m_name("")
     , m_dispalyText("")
+    , m_aniIconPath("")
 {
 
 }
@@ -59,3 +60,13 @@ void SoundEffectsData::setSystemSoundEffect(DDesktopServices::SystemSoundEffect
 {
     m_systemSoundEffect = newSystemSoundEffect;
 }
+
+QString SoundEffectsData::aniIconPath() const
+{
+    return m_aniIconPath;
+}
+
+void SoundEffectsData::setAniIconPath(const QString &newAniIconPath)
+{
+    m_aniIconPath = newAniIconPath;
+}
