@@ -54,6 +54,15 @@ SoundModel::SoundModel(QObject *parent)
     , m_soundEffectsModel(new SoundEffectsModel(this))
     , m_soundOutputDeviceModel(new SoundDeviceModel(this))
     , m_soundInputDeviceModel(new SoundDeviceModel(this))
+    , m_audioServerModel(new AudioServerModel(this))
+    , m_showBluetoothMode(false)
+    , m_inPutPortCount(0)
+    , m_outPutCount(0)
+    , m_outPutPortComboEnable(true)
+    , m_inPutPortComboEnable(true)
+    , m_currentBluetoothMode("")
+    , m_audioMono(false)
+    , m_bluetoothModeOpts("")
 {
     m_soundEffectMapBattery = {
         { tr("Boot up"), DDesktopServices::SSE_BootUp },
@@ -111,6 +120,61 @@ void SoundModel::setActiveinPutPort(Port *newActiveinPutPort)
     setInPutPortComboIndex(m_inPutPortCombo.indexOf(m_activeinPutPort->name() + "(" + m_activeinPutPort->cardName() + ")"));
 }
 
+void SoundModel::updatePortCombo()
+{
+    QStringList outPutPortCombo;
+    QStringList inPutPortCombo;
+
+    Port* inputActivePort = nullptr;
+    Port* outputActivePort = nullptr;
+    for (Port* port : m_ports) {
+        if (port->isEnabled()) {
+            switch (port->direction()) {
+            case Port::In:
+                inPutPortCombo.append(port->name() + "(" + port->cardName() + ")");
+                break;
+            case Port::Out:
+                outPutPortCombo.append(port->name() + "(" + port->cardName() + ")");
+                break;
+            }
+        }
+
+        if (port->isActive()) {
+            switch (port->direction()) {
+            case Port::In:
+                inputActivePort= port;
+                break;
+            case Port::Out:
+                outputActivePort = port;
+                break;
+            }
+        }
+    }
+
+    setInPutPortCombo(inPutPortCombo);
+    setOutPutPortCombo(outPutPortCombo);
+
+    if (inputActivePort)
+        setActiveinPutPort(inputActivePort);
+    if (outputActivePort)
+        setActiveOutPutPort(outputActivePort);
+}
+
+Port * SoundModel::getPortForComboIndex(int index, int portType)
+{
+    QList<Port*> ports = portType == Port::In ? m_inputPorts : m_outputPorts;
+    QStringList portCombo = portType == Port::In ? m_inPutPortCombo : m_outPutPortCombo;
+    if (portCombo.count()) {
+        for (Port* port : ports) {
+            if (port->name() + "(" + port->cardName() + ")" == portCombo.at(index)) {
+                return port;
+            }
+        }
+    }
+
+    return nullptr;
+}
+
 int SoundModel::inPutPortComboIndex() const
 {
     return m_inPutPortComboIndex;
@@ -146,6 +210,7 @@ void SoundModel::setActiveOutPutPort(Port *newActiveOutPutPort)
 {
     m_activeOutPutPort = newActiveOutPutPort;
     setOutPutPortComboIndex(m_outPutPortCombo.indexOf(m_activeOutPutPort->name() + "(" + m_activeOutPutPort->cardName() + ")"));
+    setShowBluetoothMode(m_activeOutPutPort->isBluetoothPort());
 }
 
 int SoundModel::outPutPortComboIndex() const
@@ -249,12 +314,18 @@ void SoundModel::addPort(Port *port)
 
         if (port->direction() == Port::Out) {
             m_outputPorts.append(port);
+            setOutPutCount(static_cast<int>(m_outputPorts.count()));
+            if (port->isEnabled()) {
+                m_outPutPortCombo.append(port->name() + "(" + port->cardName() + ")");
+            }
 
-            m_outPutPortCombo.append(port->name() + "(" + port->cardName() + ")");
             m_soundOutputDeviceModel->addData(port);
         } else {
             m_inputPorts.append(port);
-            m_inPutPortCombo.append(port->name() + "(" + port->cardName() + ")");
+            setInPutPortCount(static_cast<int>(m_inputPorts.count()));
+            if (port->isEnabled()) {
+                m_inPutPortCombo.append(port->name() + "(" + port->cardName() + ")");
+            }
             m_soundInputDeviceModel->addData(port);
             emit inPutPortComboChanged();
         }
@@ -273,12 +344,14 @@ void SoundModel::removePort(const QString &portId, const uint &cardId)
 
         if (port->direction() == Port::Out) {
             m_outputPorts.removeOne(port);
+            setOutPutCount(static_cast<int>(m_outputPorts.count()));
             m_outPutPortCombo.removeOne(port->name() + "(" + port->cardName() + ")");
 
             m_soundOutputDeviceModel->removeData(port);
 
         } else {
             m_inputPorts.removeOne(port);
+            setInPutPortCount(static_cast<int>(m_inputPorts.count()));
             m_inPutPortCombo.removeOne(port->name() + "(" + port->cardName() + ")");
             m_soundInputDeviceModel->removeData(port);
             emit inPutPortComboChanged();
@@ -496,6 +569,7 @@ void SoundModel::updateSoundEffectsModel()
             data->setSystemSoundEffect(item.second);
             data->setChecked(m_soundEffectData[item.second]);
             data->setPath(m_soundEffectPaths[item.second]);
+            data->setAniIconPath("");
             m_soundEffectsModel->addData(data);
         }
     }
@@ -552,6 +626,76 @@ void SoundModel::setOutPutCount(int newOutPutCount)
     emit outPutCountChanged();
 }
 
+void SoundModel::updatePlayAniIconPath(int index, const QString &newPlayAniIconPath)
+{
+    m_soundEffectsModel->updateSoundEffectsAniIcon(index, newPlayAniIconPath);
+}
+
+AudioServerModel *SoundModel::audioServerModel() const
+{
+    return m_audioServerModel;
+}
+
+void SoundModel::setAudioServerModel(AudioServerModel *newAudioServerModel)
+{
+    m_audioServerModel = newAudioServerModel;
+}
+
+void SoundModel::addAudioServerData(const AudioServerData &newAudioServerData)
+{
+    m_audioServerModel->addData(newAudioServerData);
+}
+
+bool SoundModel::audioMono() const
+{
+    return m_audioMono;
+}
+
+void SoundModel::setAudioMono(bool newAudioMono)
+{
+    m_audioMono = newAudioMono;
+    emit audioMonoChanged();
+}
+
+bool SoundModel::showBluetoothMode() const
+{
+    return m_showBluetoothMode;
+}
+
+void SoundModel::setShowBluetoothMode(bool newShowBluetoothMode)
+{
+    if (m_showBluetoothMode == newShowBluetoothMode)
+        return;
+    m_showBluetoothMode = newShowBluetoothMode;
+    emit showBluetoothModeChanged();
+}
+
+bool SoundModel::outPutPortComboEnable() const
+{
+    return m_outPutPortComboEnable;
+}
+
+void SoundModel::setOutPutPortComboEnable(bool newOutPutPortComboEnable)
+{
+    if (m_outPutPortComboEnable == newOutPutPortComboEnable)
+        return;
+    m_outPutPortComboEnable = newOutPutPortComboEnable;
+    emit outPutPortComboEnableChanged();
+}
+
+bool SoundModel::inPutPortComboEnable() const
+{
+    return m_inPutPortComboEnable;
+}
+
+void SoundModel::setInPutPortComboEnable(bool newInPutPortComboEnable)
+{
+    if (m_inPutPortComboEnable == newInPutPortComboEnable)
+        return;
+    m_inPutPortComboEnable = newInPutPortComboEnable;
+    emit inPutPortComboEnableChanged();
+}
+
 void SoundModel::setInPutPortCount(int newInPutPortCount)
 {
     if (m_inPutPortCount == newInPutPortCount)
@@ -602,16 +746,6 @@ SoundEffectsModel* SoundModel::soundEffectsModel() const
     return m_soundEffectsModel;
 }
 
-QString SoundModel::getListName(int index) const
-{
-    if (index == 0) {
-        return "zzzzzzzzzzzzz";
-    } else if (index == 1) {
-        return "hhhhhhhhhhhhhhh";
-    } else {
-        return "ffffffffffffffffff";
-    }
-}
 
 int SoundModel::getSoundEffectsRowCount() const
 {
@@ -623,6 +757,8 @@ void SoundModel::setAudioServer(const QString &audioServer)
     if (m_audioServer != audioServer) {
         m_audioServer = audioServer;
         Q_EMIT curAudioServerChanged(audioServer);
+
+        m_audioServerModel->updateCheckedService(audioServer);
     }
 }
 
