@@ -3,17 +3,24 @@
 // SPDX-License-Identifier: GPL-3.0-or-later
 #include "soundworker.h"
 
-#include <QJsonDocument>
-#include <QTimer>
-#include <QJsonArray>
-#include <QJsonObject>
+#include <DGuiApplicationHelper>
+
+#include <QAudioDevice>
 #include <QDebug>
+#include <QJsonArray>
+#include <QJsonDocument>
+#include <QJsonObject>
 #include <QLoggingCategory>
+#include <QMediaPlayer>
+#include <QPair>
+#include <QTimer>
 
 Q_LOGGING_CATEGORY(DdcSoundWorker, "dcc-sound-worker")
 
 #define GSETTINGS_WAIT_SOUND_RECEIPT "wait-sound-receipt"
 
+const QList<QPair<QString, QString>>& AudioServerNames = {qMakePair(QString("pipewire"),QString("PipeWire")), qMakePair(QString("pulseaudio"),QString("PulseAudio"))};
+
 SoundWorker::SoundWorker(SoundModel *model, QObject *parent)
     : QObject(parent)
     , m_model(model)
@@ -22,15 +29,45 @@ SoundWorker::SoundWorker(SoundModel *model, QObject *parent)
     , m_soundDBusInter(new SoundDBusProxy(this))
     , m_pingTimer(new QTimer(this))
     , m_inter(QDBusConnection::sessionBus().interface())
+    , m_sound(new QSoundEffect(this))
+    , m_waitInputReceiptTimer(new QTimer(this))
+    , m_waitOutputReceiptTimer(new QTimer(this))
+    , m_mediaDevices(new QMediaDevices(this))
+    , m_playAnimationTime(new QTimer(this))
+    , m_playAniIconIndex(1)
 {
     m_pingTimer->setInterval(5000);
     m_pingTimer->setSingleShot(false);
     m_waitSoundPortReceipt = 1000;
+
+    m_playAnimationTime->setInterval(300);
+    m_playAnimationTime->setSingleShot(false);
+
+    m_waitInputReceiptTimer->setSingleShot(true);
+    m_waitOutputReceiptTimer->setSingleShot(true);
+
+    m_sound->setAudioDevice(QMediaDevices::defaultAudioOutput());
+    qDebug() << " sound is playging " << m_sound->isPlaying();
     initConnect();
 }
 
 void SoundWorker::initConnect()
 {
+
+    connect(m_mediaDevices, &QMediaDevices::audioOutputsChanged,this, [this] {
+        QAudioDevice defaultDevice = QMediaDevices::defaultAudioOutput();
+        qDebug() << "audioDeviceChanged Device:" << defaultDevice.description();
+        // 默认播放设备发生变化，需要重新构建新的对象
+        if (defaultDevice.description() != m_sound->audioDevice().description()) {
+            delete m_sound;
+            m_sound = new QSoundEffect(this);
+            m_sound->setAudioDevice(defaultDevice);
+        }
+    });
+
+    connect(m_playAnimationTime, &QTimer::timeout, this, &SoundWorker::onAniTimerTimeOut);
+    connect(m_sound, &QSoundEffect::playingChanged, this, &SoundWorker::onSoundPlayingChanged);
+
     connect(m_model, &SoundModel::defaultSinkChanged, this, &SoundWorker::defaultSinkChanged);
     connect(m_model, &SoundModel::defaultSourceChanged, this, &SoundWorker::defaultSourceChanged);
     connect(m_model, &SoundModel::audioCardsChanged, this, &SoundWorker::cardsChanged);
@@ -53,6 +90,15 @@ void SoundWorker::initConnect()
 
     connect(m_soundDBusInter, &SoundDBusProxy::CurrentAudioServerChanged, m_model, &SoundModel::setAudioServer);
     connect(m_soundDBusInter, &SoundDBusProxy::AudioServerStateChanged, m_model, &SoundModel::setAudioServerChangedState);
+    connect(m_soundDBusInter, &SoundDBusProxy::AudioMonoChanged, m_model, &SoundModel::setAudioMono);
+
+    connect(m_waitInputReceiptTimer, &QTimer::timeout, this, [ this ] {
+        m_model->setInPutPortComboEnable(true);
+    });
+
+    connect(m_waitOutputReceiptTimer, &QTimer::timeout, this, [this] {
+        m_model->setOutPutPortComboEnable(true);
+    });
 }
 
 void SoundWorker::activate()
@@ -72,7 +118,9 @@ void SoundWorker::activate()
     m_model->setWaitSoundReceiptTime(m_waitSoundPortReceipt);
     m_model->setAudioServer(m_soundDBusInter->audioServer());
     m_model->setAudioServerChangedState(m_soundDBusInter->audioServerState());
+    m_model->setAudioMono(m_soundDBusInter->audioMono());
 
+    initAudioServerData();
     refreshSoundEffect();
     m_model->updateSoundEffectsModel();
 
@@ -81,7 +129,6 @@ void SoundWorker::activate()
 
     m_pingTimer->start();
     m_soundDBusInter->blockSignals(false);
-
     defaultSinkChanged(m_model->defaultSink());
     defaultSourceChanged(m_model->defaultSource());
     cardsChanged(m_model->audioCards());
@@ -103,7 +150,6 @@ void SoundWorker::refreshSoundEffect()
 void SoundWorker::setAudioServer(const QString &value)
 {
     m_soundDBusInter->SetAudioServer(value);
-    m_model->setAudioServer(value);
 }
 
 void SoundWorker::switchSpeaker(bool on)
@@ -129,9 +175,22 @@ void SoundWorker::setSinkBalance(double balance)
 
 }
 
-void SoundWorker::setActiveOutPutPort(int index)
+void SoundWorker::setActivePort(int index, int portType)
 {
+    if (portType == Port::Out) {
+        m_model->setOutPutPortComboEnable(false);
+        m_waitOutputReceiptTimer->setInterval(m_model->currentWaitSoundReceiptTime());
+        m_waitOutputReceiptTimer->start();
+    } else if(portType == Port::In) {
+        m_model->setInPutPortComboEnable(false);
+        m_waitInputReceiptTimer->setInterval(m_model->currentWaitSoundReceiptTime());
+        m_waitInputReceiptTimer->start();
+    }
 
+    Port* port = m_model->getPortForComboIndex(index, portType);
+    if (port) {
+        setPort(port);
+    }
 }
 
 void SoundWorker::setSoundEffectEnable(int index, bool enable)
@@ -219,6 +278,21 @@ void SoundWorker::setPortEnableIndex(int index, bool checked, int portType)
     }
 }
 
+void SoundWorker::playSoundEffect(int index)
+{
+    auto eff = m_model->soundEffectMap()[index].second;
+
+    if (m_sound->isPlaying()) {
+        m_sound->stop();
+        m_model->updatePlayAniIconPath(m_upateSoundEffectsIndex, "");
+    }
+    m_upateSoundEffectsIndex = index;
+
+    m_sound->setSource(QUrl::fromLocalFile(m_model->soundEffectPathByType(eff)));
+    m_sound->setVolume(1);
+    m_sound->play();
+}
+
 void SoundWorker::setBluetoothMode(const QString &mode)
 {
     m_soundDBusInter->SetBluetoothAudioMode(mode);
@@ -332,8 +406,7 @@ void SoundWorker::cardsChanged(const QString &cards)
         }
     }
 
-    m_model->setInPutPortCount(m_model->inPutPortCombo().count());
-    m_model->setOutPutCount(m_model->outPutPortCombo().count());
+    m_model->updatePortCombo();
 }
 
 void SoundWorker::activeSinkPortChanged(const AudioPort &activeSinkPort)
@@ -402,6 +475,32 @@ void SoundWorker::getSoundPathFinished(QDBusPendingCallWatcher *watcher)
     watcher->deleteLater();
 }
 
+void SoundWorker::onAniTimerTimeOut()
+{
+    QString path = QString("qrc:/icons/deepin/builtin/icons/dcc_volume%1.svg").arg(m_playAniIconIndex);
+
+    m_model->updatePlayAniIconPath(m_upateSoundEffectsIndex, path);
+    m_playAniIconIndex++;
+    if (m_playAniIconIndex > 3) {
+        m_playAniIconIndex = 1;
+    }
+}
+
+void SoundWorker::onSoundPlayingChanged()
+{
+    m_playAniIconIndex = 1;
+    QString path("");
+    if (m_sound->isPlaying()) {
+        path = QString("qrc:/icons/deepin/builtin/icons/sound_preview_%1.svg").arg(m_playAniIconIndex);
+        m_playAniIconIndex++;
+        m_playAnimationTime->start();
+    } else {
+        m_playAnimationTime->stop();
+    }
+
+    m_model->updatePlayAniIconPath(m_upateSoundEffectsIndex, path);
+}
+
 void SoundWorker::updatePortActivity()
 {
     for (Port *port : m_model->ports()) {
@@ -418,3 +517,42 @@ void SoundWorker::updatePortActivity()
         }
     }
 }
+
+void SoundWorker::initAudioServerData()
+{
+    // treeland单独处理，避免后续这里有变动
+    if (Dtk::Gui::DGuiApplicationHelper::testAttribute(Dtk::Gui::DGuiApplicationHelper::IsWaylandPlatform)) {
+        AudioServerData data;
+        data.name = "PipeWire";
+        data.serverName = "pipewire";
+        data.checked = true;
+        m_model->addAudioServerData(data);
+        return;
+    }
+
+    for (auto item : AudioServerNames) {
+        AudioServerData data;
+        data.name = item.second;
+        data.serverName = item.first;
+
+        data.checked = false;
+        if (data.serverName == m_model->audioServer()) {
+            data.checked = true;
+        }
+        m_model->addAudioServerData(data);
+    }
+}
+
+void SoundWorker::setAudioServerIndex(int index)
+{
+    if (index >= 0  && AudioServerNames.count() > index) {
+        setAudioServer(AudioServerNames.at(index).first);
+    }
+}
+
+void SoundWorker::setAudioMono(bool enable)
+{
+    if (enable != m_model->audioMono()) {
+        m_soundDBusInter->setAudioMono(enable);
+    }
+}
